<?php

// Exit if accessed directly
if ( ! defined( 'ABSPATH' ) ) exit;


//-----------------------------------------------
// get, update, delete option
//-----------------------------------------------

function qrr_get_option( $key = '', $default = false ) {
    global $qrr_options;
    $value = ! empty( $qrr_options[ $key ] ) ? $qrr_options[ $key ] : $default;
    $value = apply_filters( 'qrr_get_option', $value, $key, $default );
    return apply_filters( 'qrr_get_option_' . $key, $value, $key, $default );
}

function qrr_update_option( $key = '', $value = false ) {

    if ( empty( $key ) ){
        return false;
    }

    if ( empty( $value ) ) {
        $remove_option = qrr_delete_option( $key );
        return $remove_option;
    }

    $options = get_option( 'qrr_settings' );

    $value = apply_filters( 'qrr_update_option', $value, $key );

    $options[ $key ] = $value;
    $did_update = update_option( 'qrr_settings', $options );

    if ( $did_update ){
        global $qrr_options;
        $qrr_options[ $key ] = $value;

    }

    return $did_update;
}

function qrr_delete_option( $key = '' ) {

    if ( empty( $key ) ){
        return false;
    }

    $options = get_option( 'qrr_settings' );

    // Next let's try to update the value
    if( isset( $options[ $key ] ) ) {
        unset( $options[ $key ] );
    }

    $did_update = update_option( 'qrr_settings', $options );

    if ( $did_update ){
        global $qrr_options;
        $qrr_options = $options;
    }

    return $did_update;
}

function qrr_get_settings() {

    $settings = get_option( 'qrr_settings' );

    if( empty( $settings ) ) {

        // Set default settings here

        //update_option( 'qrr_settings', $settings );
    }

    return apply_filters( 'qrr_get_settings', $settings );
}



//-----------------------------------------------
// Register Settings
//-----------------------------------------------

function qrr_register_settings() {

    if ( false == get_option( 'qrr_settings' ) ) {
        add_option( 'qrr_settings' );
    }

    foreach( qrr_get_registered_settings() as $tab => $settings ) {

        // Manage tab with several sections inside (name_) underscore at the end means sub settings
        if (preg_match('/.+_$/', $tab)) {

            foreach($settings as $sec=>$sub_settings) {
                qrr_add_settings_section( $tab.$sec );
                qrr_add_settings_fields( $tab.$sec, $sub_settings );
            }
        }
        else {
            qrr_add_settings_section( $tab);
            qrr_add_settings_fields( $tab, $settings );
        }
    }

    register_setting( 'qrr_settings', 'qrr_settings', 'qrr_settings_sanitize' );
}
add_action( 'admin_init', 'qrr_register_settings' );



function qrr_add_settings_section( $tab ) {

    add_settings_section(
        'qrr_aettings' . '_' . $tab,
        __return_null(),
        '__return_false',
        'qrr_settings' . '_' . $tab
    );

}

function qrr_add_settings_fields( $tab, $settings ) {

    foreach( $settings as $key => $option ) {

        $name = isset( $option['name'] ) ? $option['name'] : '';

        $callback = function_exists( 'qrr_settings_show_'.$option['type'] ) ? 'qrr_settings_show_'.$option['type'] : 'qrr_settings_show_missing';

        add_settings_field(
            'qrr_settings' . '[' . $key . ']',
            $name,
            $callback,
            'qrr_settings'. '_' . $tab, // Page
            'qrr_settings' . '_' . $tab, // Section
            array(
                'id'      => $key,
                'desc'    => ! empty( $option['desc'] ) ? $option['desc'] : '',
                'name'    => isset( $option['name'] ) ? $option['name'] : null,
                'section' => $tab,
                'size'    => isset( $option['size'] ) ? $option['size'] : null,
                'options' => isset( $option['options'] ) ? $option['options'] : '',
                'std'     => isset( $option['std'] ) ? $option['std'] : '',
                'callback'=> isset( $option['callback'] ) ? $option['callback'] : '',
                'options' => isset( $option['options'] ) ? $option['options'] : ''
            )
        );
    }
}


function qrr_get_registered_settings(){

    $list = array(
        'license' => apply_filters( 'erm_settings_license',
            array(
                'erm_pro_license_key' => array(
                    'name' => __( 'License key', 'erm' ),
                    'desc' => __( 'Enter your license key.' , 'erm' ),
                    'type' => 'text',
                    'size' => 'regular',
                    'callback' => 'qrr_sanitize_license'
                ),
            )
        ),
        'general' => apply_filters( 'erm_settings_general', // Tab general
            array(
                'erm_currency' => array(
                    'name' => __( 'Currency', 'erm' ),
                    'desc' => __( 'Add this character to the price. Leave it blank if you don\'t want to display.' , 'erm' ),
                    'type' => 'text',
                    'size' => 'small',
                    'std' => ''
                ),
                'erm_currency_position' => array(
                    'name' => __( 'Currency position', 'erm' ),
                    'desc' => __( '' , 'erm' ),
                    'type' => 'radio',
                    'size' => 'regular',
                    'std' => 'before',
                    'options' => array(
                        'before' => __('Before price', 'erm'),
                        'after' => __('After price', 'erm')
                    )
                ),
                'erm_show_dashboard_menu_items' => array(
                    'name' => __( 'Show Menu Items', 'erm' ),
                    'desc' => __( 'Display the Dashboard MENU for post type "Menu Items"' , 'erm' ),
                    'type' => 'checkbox',
                    'size' => '',
                    'std' => ''
                ),
                /*'erm_menu_thumb_size' => array(
                    'name' => __( 'Menu items thumbnail size', 'erm' ),
                    'desc' => __( '' , 'erm' ),
                    'type' => 'select_size',
                    'size' => 'regular',
                    'std' => ''
                ),*/
                'erm_custom_css' => array(
                    'name' => __( 'Custom CSS', 'erm' ),
                    'desc' => __( '' , 'erm' ),
                    'type' => 'textarea',
                    'size' => 'large',
                    'std' => ''
                ),
                'erm_custom_css_display' => array(
                    'name' => __( 'Insert Custom CSS', 'erm' ),
                    'desc' => __( 'Inject the CSS in the Front End' , 'erm' ),
                    'type' => 'checkbox',
                    'size' => '',
                    'std' => ''
                )

            )
        )
    );

    return apply_filters( 'qrr_registered_settings', $list );
}



//-----------------------------------------------
// Show settings
//-----------------------------------------------

function qrr_settings_show_missing(){
    return 'MISSING FIELD';
}










//-----------------------------------------------
// Sanitize settings
//-----------------------------------------------

function qrr_settings_sanitize( $input ) {

    $output = $input;

    return $output;
}

